@@ -3,7 +3,7 @@
 //!
 //! ```toml
 //! [dependencies]
-//! lobby = "0.1"
+//! lobby-queue = "0.2"
 //! ```
 //!
 //! ```
@@ -16,10 +16,19 @@
 //! m.push(2);
 //! assert_eq!(Some(&0), m.first());
 //!
-//! m.push(3);
+//! let v0 = m.push(3);
+//! assert_eq!(Some(0), v0);
 //! assert_eq!(Some(&1), m.first());
+//!
+//! for v in m {
+//!     println!("{}", v);
+//! }
 //! ```
 
+pub mod iter;
+
+use crate::iter::{IntoIter, Iter, IterMut};
+
 use std::mem;
 
 /// A const-size queue-like data structure.
@@ -320,11 +329,13 @@ impl<T, const N: usize> Lobby<T, N> {
         Self::mod_incr(self.tail, 1) == self.head
     }
 
+    /// Returns the current number of items stored in the lobby.
     #[inline]
     pub const fn len(&self) -> usize {
         self.len
     }
 
+    /// Returns the capacity of the lobby, which will always be `N`.
     #[inline]
     pub const fn capacity(&self) -> usize {
         N
@@ -367,20 +378,16 @@ where
 }
 
 impl<T, const N: usize> Lobby<T, N> {
+    /// Returns an iterator over the items in the lobby.
     #[inline]
     pub fn iter(&self) -> Iter<'_, T, N> {
-        Iter {
-            inner: &self,
-            idx: 0,
-        }
+        Iter::new(self)
     }
 
+    /// Returns an iterator that allows modifying each value.
     #[inline]
     pub fn iter_mut(&mut self) -> IterMut<'_, T, N> {
-        IterMut {
-            inner: &mut self.arr,
-            idx: 0,
-        }
+        IterMut::new(self)
     }
 }
 
@@ -390,136 +397,7 @@ impl<T, const N: usize> IntoIterator for Lobby<T, N> {
 
     #[inline]
     fn into_iter(self) -> IntoIter<T, N> {
-        IntoIter { inner: self }
-    }
-}
-
-#[derive(Debug)]
-pub struct Iter<'a, T, const N: usize> {
-    inner: &'a Lobby<T, N>,
-
-    idx: usize,
-}
-
-impl<'a, T, const N: usize> Iter<'a, T, N> {
-    #[inline]
-    pub fn with_full(self) -> IterFull<'a, T, N> {
-        IterFull {
-            inner: self.inner,
-            idx: self.idx,
-        }
-    }
-}
-
-impl<'a, T, const N: usize> Iterator for Iter<'a, T, N> {
-    type Item = &'a T;
-
-    #[inline]
-    fn next(&mut self) -> Option<Self::Item> {
-        self.idx += 1;
-        if self.idx > N {
-            None
-        } else {
-            self.inner.nth(self.idx - 1)
-        }
-    }
-}
-
-#[derive(Debug)]
-pub struct IterFull<'a, T, const N: usize> {
-    inner: &'a Lobby<T, N>,
-
-    idx: usize,
-}
-
-impl<'a, T, const N: usize> Iterator for IterFull<'a, T, N> {
-    type Item = Option<&'a T>;
-
-    #[inline]
-    fn next(&mut self) -> Option<Self::Item> {
-        self.idx += 1;
-        if self.idx > N {
-            None
-        } else {
-            Some(self.inner.nth(self.idx - 1))
-        }
-    }
-}
-
-#[derive(Debug)]
-pub struct IterMut<'a, T, const N: usize> {
-    inner: &'a mut [Option<T>; N],
-
-    idx: usize,
-}
-
-impl<'a, T, const N: usize> IterMut<'a, T, N> {
-    #[inline]
-    pub fn with_full(self) -> IterMutFull<'a, T, N> {
-        IterMutFull { iter: self }
-    }
-}
-
-impl<'a, T, const N: usize> Iterator for IterMut<'a, T, N> {
-    type Item = &'a mut T;
-
-    #[inline]
-    fn next(&mut self) -> Option<Self::Item> {
-        if self.idx >= N {
-            None
-        } else {
-            // SAFETY: self.idx is guaranteed to be within bounds.
-            let v = (unsafe { &mut *self.inner.as_mut_ptr().add(self.idx) }).as_mut();
-            self.idx += 1;
-            v
-        }
-    }
-}
-
-#[derive(Debug)]
-pub struct IterMutFull<'a, T, const N: usize> {
-    iter: IterMut<'a, T, N>,
-}
-
-impl<'a, T, const N: usize> Iterator for IterMutFull<'a, T, N> {
-    type Item = Option<&'a mut T>;
-
-    #[inline]
-    fn next(&mut self) -> Option<Self::Item> {
-        self.iter.next().map(Some)
-    }
-}
-
-#[derive(Debug)]
-pub struct IntoIter<T, const N: usize> {
-    inner: Lobby<T, N>,
-}
-
-impl<T, const N: usize> Iterator for IntoIter<T, N> {
-    type Item = T;
-
-    fn next(&mut self) -> Option<Self::Item> {
-        self.inner.shift()
-    }
-}
-
-impl<'a, T, const N: usize> IntoIter<T, N> {
-    #[inline]
-    pub fn with_full(self) -> IntoIterFull<T, N> {
-        IntoIterFull { inner: self.inner }
-    }
-}
-
-#[derive(Debug)]
-pub struct IntoIterFull<T, const N: usize> {
-    inner: Lobby<T, N>,
-}
-
-impl<T, const N: usize> Iterator for IntoIterFull<T, N> {
-    type Item = Option<T>;
-
-    fn next(&mut self) -> Option<Self::Item> {
-        Some(self.inner.shift())
+        IntoIter::new(self)
     }
 }
 
